from goody import irange

class TN:
    def __init__(self, value, children = []):
        self.value     = value
        self.children  = children # or {} if order of children is not important


def size(atree): 
    count = 1
    for c in atree.children:
        count += size(c)
    return count
#     return 1+sum( (size(c) for c in atree.children) )


def height(atree):
    h = -1
    for c in atree.children:
        h = max(h,height(c))
    return 1 + h
#     if atree.children == []:
#         return 0
#     else:
#         return 1+max( (height(c) for c in atree.children) )


def list_to_tree(alist):
    if alist == []:
        return []
    else:
        return TN(alist[0], [list_to_tree(x) for x in alist[1:]])

    
def tree_to_list(atree): 
    return [atree.value]+[tree_to_list(c) for c in atree.children]

    
def print_tree(atree,indent_char ='.',indent_delta=2):
    def print_tree_1(indent,atree):
        if atree == None:
            return None
        else:
            children = len(atree.children)
            for c in atree.children[0:children//2]:
                print_tree_1(indent+indent_delta,c)
            print(indent*indent_char+str(atree.value))
            for c in atree.children[children//2:]:
                print_tree_1(indent+indent_delta,c)
    print_tree_1(0,atree) 

 
def contains(atree,value):
    return atree.value == value or any((contains(c,value) for c in atree.children))


def find(atree,value):
    if atree.value == value:
        return atree
    else:
        for c in atree.children:
            answer = find(c,value)
            if answer != None:
                return answer
        return None

    
def ancestors(atree,value):
    if atree.value == value:
        return [value]
    else:
        for c in atree.children:
            answer = ancestors(c,value)
            if answer != []:
                return [atree.value] + answer
        return []


def generator_pre_order(atree):
    yield atree.value
    for c in atree.children:
        yield from generator_pre_order(c)
#         for v in generator_pre_order(c):
#             yield v


def generator_post_order(atree):
    for c in atree.children:
        yield from generator_post_order(c)
#         for v in generator_post_order(c):
#             yield v
    yield atree.value


def generator_breadth_first(atree):
    to_yield = [atree]
    while to_yield != []:
        node = to_yield.pop(0)
        # to_yield += [c for c in node.children]
        for c in node.children:
            to_yield.append(c)
        yield node.value

if __name__ == '__main__':
    import prompt,traceback
    t = list_to_tree(
      ['Bands',
        ['Beach_Boys',
           ['Brian'], ['Dennis'], ['Carl'], ['Mike'], ['Al']
        ],
        ['Beatles',
           ['John'], ['Paul'], ['George'], ['Ringo']
        ],
        ['Roches',
           ['Maggie'], ['Terre'], ['Suzzy']
        ]
       ])
    print_tree(t)
    while True:
        try:
            exec(prompt.for_string('Command'))
        except Exception:
            traceback.print_exc()   